<!--
Copyright 2022 by Jonas Lache <jonas.lache@ruhr-uni-bochum.de>
Copyright 2022 by Daniel Meißner <daniel.meissner-i4k@ruhr-uni-bochum.de>
SPDX-License-Identifier: GPL-3.0-or-later
-->

# STACKrate

STACKrate is a small JavaScript module that allows to embed star
ratings in [STACK](https://stack-assessment.org) questions.  The
rating results are stored together with the student answers.

## Quickstart

You can follow these three steps to add a star rating to your STACK
question.

1. Make sure to [activate the Moodle plain text editor](#working-with-html-in-the-moodle-text-editor).
2. Add the JavaScript module and a hidden input field for the results
   to the question text:

   ```html
   <p class="ratingResults">[[input:ans_survey]] [[validation:ans_survey]]</p>
   <script src="https://rub.de/stackrate-maths/stackrate@0.1/stars.js"></script>
   ```

   Make sure to set the input type to _string_ and disable validation.
3. Place your star rating at the appropiate location (question text or
   feedback text).  It could look like this:

   ```html
   <div class="stars-rating" data-name="my_id">
     <p>Please answer the following questions:</p>
     <div class="question">
       How helpful was this feedback?
       <div class="stars"></div>
     </div>
     <div class="question">
       How helpful was the hint?
       <div class="stars"></div>
     </div>
     <div class="question">
       How would you rate the task overall?
       <div class="stars"></div>
     </div>
   </div>
   ```

For more details on the settings, see the following sections.

## Prerequisites

### Hosting your own version of STACKrate

While you can definitely refer to the JavaScript module hosted on our
infrastructure (see Step 2 in [Quickstart](#quickstart)), we do
recommend that you host STACKrate on your own infrastructure.  This
way you can ensure that STACKrate stays available under a URL of your
choice.

In order to do this, you must copy at least the following files to a
folder (we recommend the name _stackrate_) on your webspace:

```
.
├── LICENSES
│   └── GPL-3.0-or-later.txt
├── stars.css
├── stars.js
└── translations
    ├── de.json
    ├── de.json.license
    ├── es.json
    ├── es.json.license
    ├── fr.json
    ├── fr.json.license
    ├── nl.json
    ├── nl.json.license
    ├── pt.json
    └── pt.json.license
```

Of course, you can also copy the whole package to your folder.

**Note:** If you load the script from a different site than the one on which it
is hosted, also make sure that you set the `Access-Control-Allow-Origin`
response header for the JSON files in translations since our script fetches them
dynamically.  This is due to the [CORS
protocol](https://fetch.spec.whatwg.org/#http-cors-protocol).  If you host the
script using the [Apache HTTP server](https://httpd.apache.org/), you could
achieve this by placing an _.htaccess_ file with the following content into you
_stackrate_ folder (replace _example.org_ with the domain of your site):

```
<FilesMatch "\.(json)$">
  <IfModule mod_headers.c>
    Header set Access-Control-Allow-Origin "example.org"
  </IfModule>
</FilesMatch>
```

### Working with HTML in the Moodle text editor

We recommend using the Moodle text editor "plain text area" for this purpose. It
can be activated by clicking on the arrow next to the user image in the upper
right corner and then on "Preferences". Then under "User account" you can click
on "Editor preferences". Here you select "Plain text area" from the dropdown
list.

![](screenshots/plain-area-editor.jpeg)

If you prefer to use the default Moodle text editor, which is usually the
"Atto Editor", click the arrow located on the left. The toolbar of the editor
now shows more icons. Select the `</>` icon which switches to the HTML source
view.

![](screenshots/atto-editor.jpeg)

### The JavaScript module and the hidden input field

To use the star rating, it is necessary to paste the following HTML
fragment at the end of the question text of the STACK question:

```html
<p class="ratingResults">[[input:ans_survey]] [[validation:ans_survey]]</p>
<script src="https://rub.de/stackrate-maths/stackrate@0.1/stars.js"></script>
```

The first line contains a placeholder for a hidden input field.  The
rating results are written to this hidden input field and thus stored
together with the students' other answers within the Moodle database.
Note that using the name "ans_survey" means that no variable with this
name can be used within the STACK task anymore.  It is necessary to
set the input type of this hidden input field to _string_ and disable
validation.

![](screenshots/input-field.jpeg)

### Recommended settings for the Moodle quiz

If the question behaviour "Adaptive mode" is activated in the settings of the
test, the star rating can be placed in the feedback of a potential response tree
and is not visible until the students have answered the STACK question. When the
question behaviour "Deferred feedback" or "Immediate feedback" is selected, the
star ratings have to be placed in the question text and are visible for students
when they work on the task. Thus, we recommend using the "Adaptive mode"
question behaviour.

![](screenshots/quiz-settings.jpeg)

If you work with the immediate feedback question behaviour, you might want to
prevent the script from renaming the Moodle check button to "Submit rating"
which could be a bit confusing for students since it also submits their attempt.
This can be done by adding the `data-moodle-integration` attribute on the
`div.stars-rating` element to `false` like so:

```html
<div class="stars-rating" data-moodle-integration="false">
...
</div>
```

## Customizing your star rating

To create a set of evaluation questions that can be answered by using stars, the
following code must be added at the position where it should appear (question
text or feedback text):

```html
<div class="stars-rating" data-name="my_id">
    ...
</div>
```

To match the ratings which are saved to the Moodle database with the set of
evaluation questions to which they belong, choose a name and set the `data-name`
attribute accordingly. The name must consist of lowercase letters, numbers or an
underscore.

To add an evaluation question, insert the following code  between these `<div>`
tags:

```html
<div class="question">
    – Insert your question here –
    <div class="stars"></div>
</div>
```

Instead of the placeholder *– Insert your question here –*, add your own
evaluation question. It is recommended to start with an instruction like "Please
answer the following questions:" that can be added between two `<p>` elements.

Altogether, the completed star rating could look like this:

```html
<div class="stars-rating" data-name="my_id">
    <p>Please answer the following questions:</p>
    <div class="question">
        How helpful was this feedback?
        <div class="stars"></div>
    </div>
    <div class="question">
        How helpful was the hint?
        <div class="stars"></div>
    </div>
    <div class="question">
        How would you rate the task overall?
        <div class="stars"></div>
    </div>
</div>
```

This is how the rating could look like on the Moodle page:
![](screenshots/stars_rating.jpeg)

### Add a text area for comments

If you want your students to explain their rating in more details, you
could add a text area for comments. Add the class `with-comment` to
the `<div>` with the class `stars-rating` like so:

```html
<div class="stars-rating with-comment" data-name="my_id">
    ...
</div>
```

This is how the rating could look like on the Moodle page:
![](screenshots/stars_rating_comment.jpeg)

### Changing the language

By default, automatically generated text snippets like the legend and the
success message are in English. To use a different language, the `lang`
attribute must be changed at the `<div>` with the class `stars-rating`,
for example,

```html
<div class="stars-rating" data-name="my_id" lang="de">
    ...
</div>
```

where `de` stands for German.

At the moment, we support `de` (German), `es` (Spanish),
`fr` (French), `nl` (Dutch), `pt` (Portuguese), and English as the
default language.

### Changing the appearance

To change the appearance of the STACKrate box, the attribute `style` can be
added to the `<div>` element with class `stars-rating`. Via this attribute, the CSS
properties of the box can be changed. Among others, the following options are
available:

- `background-color: <color>` sets the background colour of the box where
	`<color>` is a colour name (e.g., `red`) or a colour code (e.g.,
	`#FF0000`). More information about colours can be found under [this
	link](https://developer.mozilla.org/en-US/docs/Learn/CSS/Building_blocks/Values_and_units#color).

- `color: <color>` sets the colour of the text inside the box.

- `border: <width> <color> <border-style>` sets the properties
	of the border of the box. `<stroke-width>` defines the width of the border
	(e.g., `1px`). `<border-style>` (e.g., `dashed`) defines if the border
	should be solid or dashed.

- `box-shadow: <definition-of-shadow>`: Sets the shadow of the box. The
	shadow can be disabled by choosing `box-shadow:none`. If you want to use a
	custom shadow, we recommend using [this tool](https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_Backgrounds_and_Borders/Box-shadow_generator).

The following example results in a box with a light green background, dark blue
text and no shadow (see image below):

```html
<div class="stars-rating" data-name="my_id"
	style="background-color:lightgreen; color:#00008B; box-shadow:none;">
    ...
</div>
```

![](screenshots/stars_rating_styled.jpeg)

If you want more extensive customization, you might want to set the CSS
properties for the class `stars-rating` directly instead of overwriting them
with a style attribute locally.  You could do so with a `style` element inside
of the question text or you could even host some CSS file somewhere and include
it with a link tag.  This is especially helpful if you want to create a lot of
star ratings that should look the same and you do not want to copy and paste all
the boilerplate code.


## Creating multiple sets of star rating questions

If you want to use multiple sets of star ratings in the same STACK question,
you just have to choose different `data-name` attributes for each of the sets:

```html
<div class="stars-rating" data-name="my_id_1">
    ...
</div>

<div class="stars-rating" data-name="my_id_2">
    ...
</div>
```

## Using the frontend

When students are provided with a set of star rating questions, they can select
from one to five stars. When no star is selected, the evaluation question counts
as not answered. When they selected the stars for each rating of a set, the
Moodle "Check" button changes its name to "Submit rating" unless the attribute
`data-moodle-integration` is set to `false`. When at least one star rating
question is not answered and the "Check" button is clicked, the text "Please
submit all ratings!" is shown. Students cannot continue with the exercise until
they have completed the rating. When all ratings are given and the students have
clicked on "Check" (or "Submit rating"), the set of star rating questions
disappears and the success message "Thanks for your rating!" appears instead.

## Getting the results of a rating

### How to access the results

The results of the rating are to be found within the test data for the Moodle
test. A table with all of the students' responses (including the results from
the rating) can be downloaded (Quiz Actions Menu → Results → Responses →
Download table data as Comma separated values (.csv)).

Here it is important that the exported table contains (only) the ratings you are
interested in. For example, if you want to include only the last attempt of each
student, you have to select the checkbox "Show at most one finished attempt per
user" in the section "What to include in the report" and select "Last try" from
the dropdown list "Which attempts".

![](screenshots/export-data.jpeg)

### How the results are saved

When the students answer the star rating questions, the results are copied into
the hidden STACK input field. Here, the JSON format is used. This means that the
results will be written in the following format:

```
{
    my_id_1: {
        "ratings": [3,4,2,...],
        "comment": "Comment inside the text area",
        "submitted": true/false
    },
    my_id_2: {
        ...
    },
    ...
}
```

### Using Python to extract and process the results

STACKrate includes the Python script `stars.py` with which the results of
an evaluation can be extracted from the CSV file that Moodle provides.
Please note that the script only works for CSV files where only one STACKrate
evaluation is included. We plan to release an updated version of the script
soon.

To run the script, follow these steps:

1. Install [Python 3](https://python.org/downloads) on your computer.

2. Download the script `stars.py` from our website or repository.

3. Open the file with a Python launcher. When using Windows, you probably just
	have to double-click on the file. When using macOS, you might have to
	right-click on the file to open it with the app "Python Launcher".

4. Next, a file dialog appears. Select the CSV file you downloaded from Moodle.

5. Then, another file dialog appears. Select the folder and the wished file name
	for the export file (e.g., `output.csv`).

Python has now generated two files. The one with the file name you specified in
step 5 (e.g., `output.csv`) is a table with given numbers of stars for each of
the evaluation questions of the evaluation and all the comments, for example:

| Question 1 | Question 2 | Comment                              |
|------------|------------|--------------------------------------|
| 5          | 4          |                                      |
| 5          | 5          | Good!                                |
| 1          | 2          | The feedback could be more detailed. |
| ...        | ...        | ...                                  |

The file with the supplement `_stats` (e.g., `output_stats.csv`) includes some
basic statistics on your evaluation, for example, the number of ratings, the
mean of stars given for each evaluation question, and the distribution of the
number of stars given for each evaluation question.
